#ifndef CAMERA_DRIVING_CLASS_H
#define CAMERA_DRIVING_CLASS_H

#ifndef CONF_SOC_A
#define CONF_SOC_A
#endif

#include <iostream>
#include <fstream>
#include <thread>
#include <cstring> // for strcpy

#include <publisher.h>
#include <sys_ctr.h>
#include "ara/log/logging.h"
#include "ara/exec/execution_client.h"
#include "camera_drv/executable.h"

#include "cam_preprocess.h"
#include "mdc_cam.h"
#include "camera_pool.h"
#include "timer_utils.h"
#include "MbufModule.h"

#include "CameraDriving_GeneralWrapper.h"
//#include "wrappers/encoder_wrapper.h"
//#include "wrappers/perception_wrapper.h"

class CameraDriving
{
public:
    CameraDriving();
    ~CameraDriving();
    std::thread pipelineThreadPool[4];
    void init(unsigned int inputImageWidth, unsigned int inputImageHeight, unsigned int outputImageWidth, unsigned int outputImageHeight, unsigned int cropTopOffset, unsigned int cropLeftOffset, unsigned int encoderGopNumber);

protected:
    ara::log::Logger &logger{ara::log::CreateLogger("C_D", "SoftwareMotion Camera Driving Module", ara::log::LogLevel::kDebug)};

private:
    unsigned int inputImageWidth, inputImageHeight, outputImageWidth, outputImageHeight, cropTopOffset, cropLeftOffset, encoderGopNumber;
    //EncoderWrapper _encoderWrapper;
    //PerceptionWrapper _perceptionWrapper;
    GeneralWrapper *gw = nullptr;

    mbuf::MbufModule mbufManager;
    MDCCamera *camera;
    std::shared_ptr<CameraPool> framePool;

    int encodeImagePublishedCounter[4];

    std::mutex m_writer_mutex1, m_writer_mutex2;
    int counter = 0;

    void sendBanner();
    void *load_file(std::string filepath, size_t &length);
    void threadsFunction(uint8_t id);
    void runPipelines(CamPreprocess &processor, uint8_t id);
};

#endif /* CAMERA_DRIVING_CLASS_H */
